/*
 * WireFrameScene.c
 *
 * Author: Samuel R. Buss
 *
 * Bug reports: Sam Buss, sbuss@ucsd.edu.
 * 
 * Base code for homework project #3, CSE 167, Fall 2004
 *
 * USAGE:   (Please keep these controls in your homework assignment.)
 *
 *    Press arrow keys to control view position.
 *	  left and right keys rotate the viewpoint left and right
 *	  Up and down keys rotate viewpoint up and down (up to 80 degrees).
 *
 *    Press "R" key to make step size bigger (if moving too slowly)
 *    Press "r" key to make step size smaller (if moving too fast).
 *
 *    Press "w" key to toggle wireframe mode on and off
 *    
 *    Press "M" , "m" to increase, decrease Mesh Resolution
 *
 *    Press "c" to toggle culling backfaces.
 *  
 *	  Press ESCAPE to exit.
 *
 */

#include <math.h>			// For math routines (such as sqrt & trig).
#include <stdio.h>
//#include <stdlib.h>		// For the "exit" function
#include <GL/glut.h>		// OpenGL Graphics Utility Library
#include "WireFrameScene.h"

// The next global variable controls the animation's state and speed.
float RotateAngle = 0.0f;		// Angle in degrees of rotation around y-axis
float Azimuth = 0.0;			// Rotated up or down by this amount
float AngleStepSize = 3.0f;		// Step three degrees at a time
const float AngleStepMax = 10.0f;
const float AngleStepMin = 0.1f;

// Some global state variables
int MeshCount = 6;				// The mesh resolution for the mushroom top
int WireFrameOn = 1;			// == 1 for wire frame mode
int CullBackFacesOn = 0;		// == 1 if culling back faces.
const int MeshCountMin = 3;


// glutKeyboardFunc is called below to set this function to handle
//		all "normal" key presses.
void myKeyboardFunc( unsigned char key, int x, int y )
{
	switch ( key ) {
	case 'c':
		CullBackFacesOn = 1-CullBackFacesOn;
		if ( CullBackFacesOn ) {
			glEnable( GL_CULL_FACE );				// Enable culling of back faces
		}
		else {
			glDisable( GL_CULL_FACE );				// Show all faces (front and back)
		}
		glutPostRedisplay();
		break;
	case 'm':
		MeshCount = (MeshCount>MeshCountMin) ? MeshCount-1 : MeshCount;
		glutPostRedisplay();
		break;
	case 'M':
		MeshCount ++;
		glutPostRedisplay();
		break;
	case 'w':
		WireFrameOn = 1-WireFrameOn;
		if ( WireFrameOn ) {
			glPolygonMode ( GL_FRONT_AND_BACK, GL_LINE );		// Just show wireframes
		}
		else {
			glPolygonMode ( GL_FRONT_AND_BACK, GL_FILL );		// Show solid polygons
		}
		glutPostRedisplay();
		break;
	case 'R':
		AngleStepSize *= 1.5;
		if (AngleStepSize>AngleStepMax ) {
			AngleStepSize = AngleStepMax;
		}
		break;
	case 'r':
		AngleStepSize /= 1.5;
		if (AngleStepSize<AngleStepMin ) {
			AngleStepSize = AngleStepMin;
		}
		break;
	case 27:	// Escape key
		exit(1);
	}
}

// glutSpecialFunc is called below to set this function to handle
//		all "special" key presses.  See glut.h for the names of
//		special keys.
void mySpecialKeyFunc( int key, int x, int y )
{
	switch ( key ) {
	case GLUT_KEY_UP:	
		Azimuth += AngleStepSize;
		if ( Azimuth>80.0f ) {
			Azimuth = 80.0f;
		}
		break;
	case GLUT_KEY_DOWN:
		Azimuth -= AngleStepSize;
		if ( Azimuth < -80.0f ) {
			Azimuth = -80.0f;
		}
		break;
	case GLUT_KEY_LEFT:
		RotateAngle += AngleStepSize;
		if ( RotateAngle > 180.0f ) {
			RotateAngle -= 360.0f;
		}
		break;
	case GLUT_KEY_RIGHT:
		RotateAngle -= AngleStepSize;
		if ( RotateAngle < -180.0f ) {
			RotateAngle += 360.0f;
		}
		break;
	}
	glutPostRedisplay();

}

/*
 * drawScene() handles the animation and the redrawing of the
 *		graphics window contents.
 */
void drawScene(void)
{
	// Clear the rendering window
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


	// Rotate the image
	glMatrixMode( GL_MODELVIEW );			// Current matrix affects objects positions
	glLoadIdentity();						// Initialize to the identity
	glTranslatef( 0.0, -2.5, -35.0 );				// Translate from origin (in front of viewer)
	glRotatef( Azimuth, 1.0, 0.0, 0.0 );			// Set Azimuth angle
	glRotatef( RotateAngle, 0.0, 1.0, 0.0 );		// Rotate around y-axis

	// Draw the base plane (white)
	glColor3f( 1.0, 1.0, 1.0 );
	glBegin( GL_QUADS );
	glVertex3f( -4.0, 0.0,  4.0 );
	glVertex3f(  4.0, 0.0,  4.0 );
	glVertex3f(  4.0, 0.0, -4.0 );
	glVertex3f( -4.0, 0.0, -4.0 );
	glEnd();

	// Draw the surface of rotation
	myDrawCosineSurface();				// You will need to write this function!

	// Draw the letter-based structure
	myDrawGeometry();					// You will need to write this function!

	// Flush the pipeline, swap the buffers
    glFlush();
    glutSwapBuffers();

}


	
void myDrawCosineSurface() 
{
	// For project #3 you need to write this routine.
	// See the homework instructions for information 
	//     on the ellipsoid's shape and position.
	
	// Remove the next five lines!  Replace them by your code for
	//		drawing the surface of revolution.
	glPushMatrix();
	glTranslatef( 2.5, 1.0, 2.5 );
	glScalef( 1.5, 1.0, 1.5 );				// Squashed sphere (ellipsiod, actually)
	glutSolidSphere( 1.0, MeshCount, MeshCount );
	glPopMatrix();

	return;

}

void myDrawGeometry() 
{
	// You will need to write this routine for Project #3.
	// In the supplied executable, I draw a "S" based on elongated
	//	ellipsoids.  These were formed using glutSolidSphere()'s,
	//  reoriented and scaled to turn the spheres into ellipsoids.

	return;

}

// Initialize OpenGL's rendering modes
void initRendering()
{
    glEnable( GL_DEPTH_TEST );	// Depth testing must be turned on

	glCullFace( GL_BACK );		// These two commands will cause backfaces to not be drawn

	// Possibly turn on culling of back faces.
	if ( CullBackFacesOn ) {
		glEnable( GL_CULL_FACE );
	}

	// Possibly turn on wireframe mode.
	if ( WireFrameOn ) {
		glPolygonMode ( GL_FRONT_AND_BACK, GL_LINE );		// Just show wireframes
	}
}

// Called when the window is resized
//		w, h - width and height of the window in pixels.
void resizeWindow(int w, int h)
{
	double aspectRatio;

	// Define the portion of the window used for OpenGL rendering.
	glViewport( 0, 0, w, h );	// View port uses whole window

	// Set up the projection view matrix: perspective projection
	// Determine the min and max values for x and y that should appear in the window.
	// The complication is that the aspect ratio of the window may not match the
	//		aspect ratio of the scene we want to view.
	w = (w==0) ? 1 : w;
	h = (h==0) ? 1 : h;
	aspectRatio = (double)w / (double)h;
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    gluPerspective( 12.0, aspectRatio, 20.0, 50.0 );

}


// Main routine
// Set up OpenGL, define the callbacks and start the main loop
int main( int argc, char** argv )
{
	glutInit(&argc, argv);

	// We're going to animate it, so double buffer 
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );

	// Window position (from top corner), and size (width% and hieght)
    glutInitWindowPosition( 10, 60 );
    glutInitWindowSize( 480, 360 );
    glutCreateWindow( "TwoCubes" );

	// Initialize OpenGL as we like it..
    initRendering();

	// Set up callback functions for key presses
	glutKeyboardFunc( myKeyboardFunc );			// Handles "normal" ascii symbols
	glutSpecialFunc( mySpecialKeyFunc );		// Handles "special" keyboard keys

	// Set up the callback function for resizing windows
    glutReshapeFunc( resizeWindow );

	// Call this for background processing
	// glutIdleFunc( myIdleFunction );

	// call this whenever window needs redrawing
    glutDisplayFunc( drawScene );

	fprintf(stdout, "Arrow keys control viewpoint.n");
	fprintf(stdout, "Press \"w\" to toggle wireframe mode.\n");
	fprintf(stdout, "Press \"c\" to toggle culling of back faces.\n");
	fprintf(stdout, "Press \"M\" or \"m\" to increase or decrease resolution of mushroom cap.\n");
	fprintf(stdout, "Press \"R\" or \"r\" to increase or decrease rate of movement (respectively).\n");

	// Start the main loop.  glutMainLoop never returns.
	glutMainLoop(  );

    return(0);	// This line is never reached.
}
